; ================================================================================================
; ISR fr DMX-Empfang
; ================================================================================================
;
; benutzte Register      : temp0, temp1, ZL, ZH, werden gesichert
; gem. benutzte Register : keine
; reservierte Register   : DMX_state, DMX_Buf_L, DMX_Buf_H
;
; zum Kopieren:
;
;.def DMX_Base_L			= R5				; Low-Byte DMX-Startadresse
;.def DMX_Base_H			= R6				; High-Byte DMX-Startadresse
;.def DMX_Buf_L			= R7				; Puffer fr Z-Pointer DMX-Routine
;.def DMX_Buf_H			= R8
;
;.def DMX_state			= R19				; (>15) Status-Flag fr DMX-Routine
;
;.equ DMX_In				= 0x00B0			; RAM-Puffer fr empfangene DMX-Daten

;.equ DMX_CHANNELS		= 3					; Zahl der zu empfangenden DMX-Kanle
;.equ DMX_STARTADRESS	= 1					; DMX-Startadresse
;
; ================================================================================================

_DMX_Receiver:

		push	temp1
		in      temp1, SREG
		push	temp1
		push	temp0		
		push	ZL
		push	ZH

		mov		ZL, DMX_Buf_L				; Pointer holen
		mov		ZH, DMX_Buf_H

		in		temp0,UCSRA					; USART Status und Daten holen
		in		temp1,UDR

		sbrc	temp0,FE					; Frame-Error: neuer Datenblock
		rjmp	_dmx_frame_error	
		
		cpi		DMX_state, 1   				; DMX-Status 1: Auf Startbyte testen
		breq	_dmx_startbyte

  		cpi 	DMX_state, 2   				; DMX-Status 2: Startadresse abfragen
		breq    _dmx_startadr
  		
 		cpi 	DMX_state, 3				; DMX-Status 3: Byte empfangen und in's RAM
  		brsh	_dmx_store_byte

		rjmp	_dmx_finish					; DMX-Status 0: Weiter auf FE warten

; ------------------------------------------------------------------------------------------------

_dmx_frame_error:

		ldi 	DMX_state, 1				; nchster Aufruf: Auf Startbyte warten
		cbi		UCSRA,FE					; FE-Flag lschen
		rjmp	_dmx_finish

; ------------------------------------------------------------------------------------------------

_dmx_startbyte:

		tst 	temp1	 					; Startbyte (0) empfangen?
		brne 	_dmx_err					; nein, Fehler
		ldi  	DMX_state, 2				; ja, nchster Aufruf: Auf Startadesse warten

		lds 	ZL, DMX_Start_L 			; Startadresse aus dem RAM holen
		lds 	ZH, DMX_Start_H

		sbr		Flags, (1<<FL_DMX)			; Flag fr DMX empfangen setzen

		rjmp	_dmx_finish

; ------------------------------------------------------------------------------------------------

_dmx_startadr:

		sbiw	ZH:ZL, 1					; Adresse runterzhlen
		brne	_dmx_finish					; Startadresse noch nicht erreicht

		ldi 	ZH, high(DMX_In)			; Adresse erreicht, Pointer auf Buffer
		ldi		ZL, low(DMX_In)
		ldi  	DMX_state, 3				; nchster Aufruf: Bytes speichern
		
; ------------------------------------------------------------------------------------------------

_dmx_store_byte:
			
		st		Z+, temp1					; empfangenes Byte speichern

		cpi		ZL, low(DMX_In+DMX_CHANNELS) ; empfangene Bytes zhlen
		brlo	_dmx_finish
		cpi		ZH, high(DMX_In+DMX_CHANNELS)
		brlo	_dmx_finish					; noch nicht alle Bytes empfangen

		clr		DMX_state					; alle Bytes empfangen, Warten auf nchsten FE

		rjmp	_dmx_finish

; ------------------------------------------------------------------------------------------------
	
_dmx_err:

	 	clr 	DMX_state					; Receiver Reset			

; ------------------------------------------------------------------------------------------------

_dmx_finish:

		mov		DMX_Buf_L, ZL				; Pointer wieder sichern
		mov		DMX_Buf_H, ZH

		pop		ZH
		pop		ZL
		pop		temp0
		pop		temp1
		out     SREG, temp1
		pop		temp1
			
		reti			

; ================================================================================================
