; ================================================================================================
; Kleiner RGB-Controller fr Tiny2313/4313 mit IR-Fernbedienung RC5 und 3 Kanlen SW-PWM
; Definitionen Register, Konstanten, etc.
; ================================================================================================

; ================================================================================================
; Register definieren
; ------------------------------------------------------------------------------------------------

.def Mathe_L			= R0					; Fr Multiplikationen etc.
.def Mathe_H			= R1

.def RND_L				= R2					; 16-Bit-Zufallszahlen Low-Byte
.def RND_H				= R3					; 16-Bit-Zufallszahlen Low-Byte

.def DMX_Buf_L			= R6					; Puffer fr Z-Pointer DMX-Routine
.def DMX_Buf_H			= R7

.def Last_PGM			= R8					; Zuletzt eingestelltes Programm
.def Last_Toggle		= R9					; Zuletzt empfangenes Toggle-Bit

.def temp0 				= R16					; (>15) Mehrzweck-Temp-Register	
.def temp1 				= R17
.def temp2 				= R18
.def temp3				= R19

.def pwm_count			= R20					; (>15) Zhler fr SW-PWM

.def rc5_timer			= R21					; (>15) Timer fr RC5-Empfang
.def rc5_temp			= R22					; (>15) Temp-Register fr RC5-Empfang
.def rc5_shift_l		= R23					; (>15) Shift-Register fr RC5-Empfang Low
.def rc5_shift_h		= R24					; (>15) Shift-Register fr RC5-Empfang High

.def Timebase			= R25					; (>15) Zhler fr Zeitbasis

.def Flags				= R26					; (>15) div. Flags

.def DMX_state			= R27					; (>15) Status-Flag fr DMX-Routine

; ================================================================================================
; Konstanten/Aliase definieren
; ------------------------------------------------------------------------------------------------

.equ XTAL				= 16000000				; Prozessortakt 16 MHz Quarz

.equ BAUD				= 250000				; Baudrate

.equ PWM_OUT			= PortD					; Ausgabeport PWM
.equ PWM_DDR			= DDRD					; DDR PWM-Port
.equ PWM_R				= 4						; Pin PWM Rot
.equ PWM_G				= 5						; Pin PWM Grn
.equ PWM_B				= 6						; Pin PWM Blau

.equ RC5_IN				= PinD					; Port fr RC5-Empfang
.equ RC5_OUT			= PortD					; fr Pullup
.equ RC5				= 1						; Pin RC5-Empfang

.equ SLED_OUT 			= PORTD					; Ausgabe-Port Status-LED
.equ SLED_IN 			= PIND					; Eingabe-Port Status-LED
.equ SLED_DDR 			= DDRD					; DDR Status-LED
.equ SLED	 			= 3						; Pin Status-LED

.equ DMX_CHANNELS		= 3						; Zahl der zu empfangenden DMX-Kanle

.equ DIP_OUT			= PORTB					; Out-Port DIP 1-8
.equ DIP_IN				= PINB					; In-Port DIP 1-8
.equ DIP9_OUT			= PORTD					; Out-Port DIP 9
.equ DIP9_IN			= PIND					; In-Port DIP 1-8
.equ DIP9				= 2						; Pin DIP 9

.equ FL_Tick			= 0						; Flag fr Zeitsegment abgelaufen
.equ FL_RC5				= 1						; Flag fr RC5-Befehl empfangen
.equ FL_RC5_TOGGLE		= 2						; RC5 Toggle-Bit
.equ FL_RC5_HOLD		= 3						; RC5 Taste gehalten
.equ FL_DMX				= 4						; DMX Frame empfangen

.equ BRI_MAX			= 31					; Maximaler Wert einstellbare Helligkeit
.equ PGM_COUNT			= 7						; Zahl der vorhanden Programme

; ================================================================================================
; RAM-Bereiche definieren
; ------------------------------------------------------------------------------------------------

.dseg

RGB_Buffer:										; RAM-Zwischenspeicher fr Farben vor Dimmung
temp_R:					.byte 1					; Rot einzeln
temp_G:					.byte 1					; Grn einzeln
temp_B:					.byte 1					; Blau einzeln
; ------------------------------------------------------------------------------------------------
rc5_device:				.byte 1					; Ausgabe RC5-Device (Bit 5 = Toggle-Bit) im RAM
rc5_command:			.byte 1					; Ausgabe RC5-Kommando im RAM
rc5_last_device:		.byte 1					; Zuletzt empfangenes Device
rc5_last_command:		.byte 1					; Zuletzt empfangenes Kommando
; ------------------------------------------------------------------------------------------------
PWM_Buffer:				.byte 3					; RAM-Puffer fr PWM-Ausgabedaten
DMX_In:					.byte DMX_CHANNELS		; RAM-Puffer fr empfangene DMX-Daten

; ------------------------------------------------------------------------------------------------
; Ab hier Speicher fr Gesamteinstellungen/einzelne Module, Werte werden im EEPROM gesichert:

Conf_Data:										; Beginn Konfigurations-Daten:
; ------------------------------------------------------------------------------------------------
; Globale Werte

PGM_Number:				.byte 1					; Programm-Nummer
Dimmer:					.byte 1					; Master-Dimmer
PGM_Speed:				.byte 1					; Geschwindigkeit aktuelles Programm
; ------------------------------------------------------------------------------------------------
; Modul Feste Farbe

FC_Colour:				.byte 1					; aktuell eingestellte feste Farbe
; ------------------------------------------------------------------------------------------------
; Modul Baumarkt-Fader

Fade_State:				.byte 1					; Fader Status
Fade_R:					.byte 1					; Fader Rot-Kanal
Fade_G:					.byte 1					; Fader Grn-Kanal
Fade_B:					.byte 1					; Fader Blau-Kanal
Fade_Speed:				.byte 1					; Fader Geschwindigkeit
; ------------------------------------------------------------------------------------------------
; Modul Blob

Blob_Colour:			.byte 1					; Aktuelle Blob-Farbnummer
Blob_Speed:				.byte 1					; Blob Geschwindigkeit
Blob_H:					.byte 1					; Blob-Helligkeit
Blob_State:				.byte 1					; Blob up oder down
; ------------------------------------------------------------------------------------------------
; Modul Feuersimulation

Fire_Speed:				.byte 1					; Kaminfeuer Geschwindigkeit
Fire_H:					.byte 1					; Kaminfeuer letzte Helligkeit

; ------------------------------------------------------------------------------------------------
; Modul DMX

DMX_Start_L:			.byte 1					; DMX-Startadresse L
DMX_Start_H:			.byte 1					; DMX-Startadresse H

; ------------------------------------------------------------------------------------------------
; Modul Zufallsfader

Random_Fade_Speed:		.byte 1					; Zufallsfader Geschwindigkeit
RF_R:					.byte 1					; Zufallsfader Rot-Wert
RF_G:					.byte 1					; Zufallsfader Grn-Wert
RF_B:					.byte 1					; Zufallsfader Blau-Wert
RF_R_MAX:				.byte 1					; Zufallsfader Rot-Wert Maximal
RF_G_MAX:				.byte 1					; Zufallsfader Grn-Wert Maximal
RF_B_MAX:				.byte 1					; Zufallsfader Blau-Wert Maximal
RF_State:				.byte 1					; Zufallsfader Status

; ------------------------------------------------------------------------------------------------
; Modul Gewiter

Lightning_Speed:		.byte 1					; Gewitter Geschwindigkeit

; ------------------------------------------------------------------------------------------------
Modules_End:									; Ende Modul-Daten
.equ Conf_Size			= (Modules_End - Conf_Data)	; Anzahl zu speichernder Bytes
; ================================================================================================
