; ================================================================================================
; Kleiner RGB-Controller fr Tiny2313 mit IR-Fernbedienung RC5 und 3 Kanlen SW-PWM
; Div. Hilfsroutinen
; ================================================================================================

; ================================================================================================
; Dimmer-Wert in temp1 mit temp0 verrechnen - Ergebnis wieder in temp0, temp1 bleibt erhalten
; ------------------------------------------------------------------------------------------------

_Dimmer:

		push	temp2

		rcall	_Multiply						; temp1 (Dimmer) mit zu dimmendem Wert multipl.
		clr		temp2
		add		Mathe_L, temp0					; noch mal dazu addieren, wegen Bereich 1-256
		adc		Mathe_H, temp2					; in 16 Bit
		mov		temp0, Mathe_H					; durch 256 dividieren
			
		pop		temp2
		ret

; ================================================================================================
; Multiplikation von temp1 und temp0, Ergebnis in Mathe_L und Mathe_H - lt. Mikrocontroller.net
; ------------------------------------------------------------------------------------------------

_Multiply:
 
		push	temp1
		push	temp2
		push	temp3

    	ldi  	temp3, 8          			; 8 mal verschieben und gegebenenfalls addieren
    	clr  	temp2             			; 0 wird fr die 16 Bit Addition bentigt
    	clr  	Mathe_L              		; Ergebnis Low Byte auf 0 setzen
    	clr  	Mathe_H              		; Ergebnis High Byte auf 0 setzen
 
mult:
    	lsl  	Mathe_L              		; 16-Bit-Ergebnis einmal nach links verschieben
    	rol  	Mathe_H
    	lsl  	temp1             			; Das MSB von temp1 ins Carry schieben
    	brcc 	noadd           			; Ist dieses MSB (jetzt im Carry) eine 1?
    	add  	Mathe_L, temp0          	; Wenn ja, dann temp0 zum Ergebnis addieren
    	adc  	Mathe_H, temp2				; in 16 Bit
 
noadd:
    	dec  	temp3            			; Wurden alle 8 Bit von Mathe2 abgearbeitet?
    	brne 	mult            			; Wenn nicht, dann ein erneuter Verschiebe/Addier Zyklus

		pop		temp3						; fertig, Register wiederherstellen
		pop		temp2
		pop		temp1
		ret									; und zurck

; ================================================================================================
; Zufallszahlen-Routine - von Mikrocontroller.net
; ------------------------------------------------------------------------------------------------
; benutzte Register      : temp0, temp1, werden gesichert
; gem. benutzte Register : keine
; reservierte Register   : RND_L, RND_H, enthalten Zufallszahl, mssen !=0 sein
; Parameter				 : keine
; ------------------------------------------------------------------------------------------------
; zum Kopieren:
;
; .def RND_L			= R3					; 16-Bit-Zufallszahlen Low-Byte
; .def RND_H			= R4					; 16-Bit-Zufallszahlen Low-Byte
;
;
;		ldi		temp0, 0b01100011			; Startwert Zufallszahlengenerator
;		mov		RND_L, temp0
;		mov		RND_H, temp0
; ------------------------------------------------------------------------------------------------

_Random:

		push	temp0
		push	temp1

 		mov		temp0, RND_L
 		andi	temp0, 0b00000001
 		lsl		temp0
 		lsl		temp0
 		eor		temp0, RND_L
 		andi	temp0, 0b00000100
 		swap	temp0
 		lsl		temp0
 		mov		temp1, RND_H
 		andi	temp1, 0b00001000
 		swap	temp1
 		eor		temp1, RND_H
 		andi	temp1, 0b10000000
 		eor		temp1, temp0
 		rol		temp1    
		rol		RND_L
		rol		RND_H

		pop		temp1
		pop		temp0
		ret

; ================================================================================================
; EEPROM_read, liest den EEPROM-Wert an der Adresse im X-Pointer und legt ihn in temp0 ab
; sonst werden keine Register benutzt
; ------------------------------------------------------------------------------------------------

_EEPROM_read:

		sbic    EECR, EEWE					; Schreibzugriff aktiv?
        rjmp    _EEPROM_read				; Ja, warten

        out		EEARL, YL                  	; Adresse laden

        sbi		EECR, EERE                 	; Lesevorgang starten
        in		temp0, EEDR

        ret

; ================================================================================================
; EEPROM_write, schreibt die Daten in temp0 ins EEPROM an die Adresse im X-Pointer
; temp0 wird anschlieend als Zwischenspeicher genutzt und berschrieben!
; ------------------------------------------------------------------------------------------------

_EEPROM_write:

        sbic    EECR, EEWE                  ; Letzter Schreibvorgang beendet...?
        rjmp    _EEPROM_write               ; Nein, warten

        out     EEARL, YL                   ; Adresse laden

        out     EEDR, temp0					; Daten  laden

        in     	temp0, SREG					; SREG sichern

        cli	                                ; Schreibzugriff muss atomar erfolgen

        sbi     EECR,EEMWE	                ; Schreiben vorbereiten
        sbi     EECR,EEWE	                ; Schreibzugriff starten
		
		sei									; Interrupts wieder aktivieren

        out     SREG, temp0                 ; SREG wieder herstellen
 
        ret

; ================================================================================================
; Konfiguration aus EEPROM lesen
; ------------------------------------------------------------------------------------------------

_Load_Config:

		push	temp0						; Register sichern
		push	temp1
		push	YL
		push	YH
		push	ZL
		push	ZH

		ldi		temp1, Conf_Size			; Zahl der zu bertragenden Bytes

		ldi		YH, High(EEP_Conf_Data)		; Zeiger auf Daten im EEPROM
		ldi		YL, Low(EEP_Conf_Data)

		ldi		ZH, High(Conf_Data)			; Zeiger auf Daten im RAM
		ldi		ZL, Low(Conf_Data)

_Load_Config_Loop:

		rcall	_EEPROM_read				; Byte aus EEPROM lesen (in temp0)

		st		Z+, temp0					; in's RAM speichern

		adiw	YH:YL, 1					; EEP-Zeiger eins weiter

		dec		temp1						; Alle Bytes bertragen?
		brne	_Load_Config_Loop			; nein, nchstes

		pop		ZH							; ja, Register wieder herstellen
		pop		ZL
		pop		YH
		pop		YL
		pop		temp1
		pop		temp0

		ret									; und zurck

; ================================================================================================
; Konfiguration ins EEPROM speichern
; ------------------------------------------------------------------------------------------------

_Save_Config:

		push	temp0						; Register sichern
		push	temp1
		push	YL
		push	YH
		push	ZL
		push	ZH

		ldi		temp1, Conf_Size			; Zahl der zu bertragenden Bytes

		ldi		YH, High(EEP_Conf_Data)		; Zeiger auf Daten im EEPROM
		ldi		YL, Low(EEP_Conf_Data)

		ldi		ZH, High(Conf_Data)			; Zeiger auf Daten im RAM
		ldi		ZL, Low(Conf_Data)

_Save_Config_Loop:

		ld		temp0, Z+					; Byte aus dem RAM holen

		rcall	_EEPROM_write				; Byte in's EEPROM schreiben

		adiw	YH:YL, 1					; EEP-Zeiger eins weiter

		dec		temp1						; Alle Bytes bertragen?
		brne	_Save_Config_Loop			; nein, nchstes

		pop		ZH							; ja, Register wieder herstellen
		pop		ZL
		pop		YH
		pop		YL
		pop		temp1
		pop		temp0

		ret									; und zurck

 ; ================================================================================================
