; ================================================================================================
; RC5-Empfangsroutine
;
; Vorlage stammt von Peter Danegger, verffentlicht im c-Forum,
; von Pesi nach Assembler bersetzt (19.11.2011)
;
; Empfnger wird regelmig gepollt, Routine muss in einer Timer-ISR laufen
; Zeiten sind angepasst fr 8 MHz Takt, Prescaler 1 = Aufruf alle 256 Takte
; FALLS DIREKT AUFGERUFEN, STATUSREGISTER SICHERN!
; ================================================================================================
;
; benutzte Register			: temp1, wird gesichert
; gem. benutzte Register 	: keine
; reservierte Register:
;.def rc5_timer			= R20				; Timer fr RC5-Empfang
;.def rc5_temp			= R21				; Temp-Register fr RC5-Empfang
;.def rc5_shift_l		= R2				; Shift-Register fr RC5-Empfang Low
;.def rc5_shift_h		= R3				; Shift-Register fr RC5-Empfang High
;
; Konstanten (zum Kopieren):
;
;.equ rc5_device			= 0x60				; Ausgabe RC5-Device (Bit 5 = Toggle-Bit) im RAM
;.def rc5_command		= 0x61				; Ausgabe RC5-Kommando im RAM
;
;.equ RC5_IN				= PinB				; Port fr RC5-Empfang
;.equ RC5_OUT			= PortB				; fr Pullup
;.equ RC5				= 0					; Pin RC5-Empfang
;
; ================================================================================================

.equ RC5_MAX			= 170				; max. Pulslnge
.equ RC5_NORM			= 100				; normale Pulslnge
.equ RC5_MIN			= 40				; min. Pulslnge

; ================================================================================================

_rc5_receiver:

		push	temp1

		inc		rc5_timer					; Timer weiterzhlen

		cpi		rc5_timer,RC5_MAX			; Maximalzeit erreicht?
		brlo	_rc5_get_bit				; Nein, Pin abfragen

		mov		temp1,rc5_shift_h			; Ja, Startbits im Schieberegister gesetzt?
		sbrs	temp1,5
		rjmp	_rc5_overrun				; nein, Timeout-Fehler

		sbrc	temp1,6						; mehr als 14 Bit empfangen..?
		rjmp	_rc5_overrun				; ja, Fehler

		rol		rc5_shift_l					; ja, komplettes Telegramm empfangen,
		rol		rc5_shift_h					; auf Device und Command aufteilen
		rol		rc5_shift_l					; 2 MSB von shift_l
		rol		rc5_shift_h					; nach shift_h bertragen

		lsr		rc5_shift_l					; shift_l wiederherstellen
		lsr		rc5_shift_l

		sts		rc5_device,rc5_shift_h		; im RAM speichern
		sts		rc5_command,rc5_shift_l

_rc5_overrun:

		clr		rc5_shift_l					; Schieberegister lschen
		clr		rc5_shift_h

_rc5_get_bit:

		in		temp1,RC5_IN				; Bitmuster RC5-Port holen
		andi	temp1,(1<<RC5)				; Nur Pin fr Empfnger interessiert
		cp		temp1,rc5_temp				; mit letztem Stand vergleichen
		breq	_rc5_receiver_ende			; keine nderung, Ende

		mov		rc5_temp,temp1				; nderung, fr nchste Abfrage speichern

		cpi		rc5_timer,RC5_MIN			; Puls lnger als Minimal ntig..?
		brsh	_rc5_check_start			; ja, auf Start prfen

		clr		rc5_shift_l					; nein, Puls zu kurz, SR lschen
		clr		rc5_shift_h

_rc5_check_start:

		clr		temp1
		cp		rc5_shift_h,temp1
		cpc		rc5_shift_l,temp1
		breq	_rc5_start					; SR leer -> Startbit empfangen

		cpi		rc5_timer,RC5_NORM			; Normale Pulslnge erreicht...?
		brlo	_rc5_receiver_ende			; Nein, Hilfsflanke empfangen, Ende

_rc5_start:

		sbrc	rc5_shift_h,6				; schon 14 Bits empfangen...?
		rjmp	_rc5_clear					; ja, Timer lschen, Ende

		clc									; Nein, SR schieben
		rol		rc5_shift_l
		rol		rc5_shift_h

		sbrc	rc5_temp,RC5				; Positive Flanke am Input?
		rjmp	_rc5_clear					; nein, Timer zurck setzen 

		sbr		rc5_shift_l, (1<<0)			; ja, Bit im SR setzen

_rc5_clear:

		clr		rc5_timer					; Timer zurck setzen

_rc5_receiver_ende:

		pop		temp1

		ret

; ================================================================================================
