; ================================================================================================
; SW-PWM-Routine
; holt Bytes aus dem RAM und gibt SW-PWM am eingestellten Port aus
; ------------------------------------------------------------------------------------------------
;
; benutzte Register      : temp0, temp1, ZL, ZH, werden gesichert
; gem. benutzte Register : keine
; reservierte Register   : pwm_count (muss high-Register sein)
;
; Konstanten (zum Kopieren):
;
;.equ PWM_BUFFER		= 0x0060			; RAM-Puffer fr PWM-Ausgabedaten
;
;.equ PWM_OUT			= PortD				; Ausgabeport PWM
;.equ PWM_DDR			= DDRD				; DDR PWM-Port
;.equ PWM_R				= 0					; Pin PWM Rot
;.equ PWM_G				= 1					; Pin PWM Grn
;.equ PWM_B				= 2					; Pin PWM Blau
;
; ================================================================================================

_sw_pwm:

		push	temp0						; Register sichern
		push	temp1
		push	ZL
		push	ZH

		inc		pwm_count					; PWM- Zhler weiter
		cpi		pwm_count, 255				; komplett durch?
		brlo	_sw_pwm_weiter				; nein, weiter
		clr		pwm_count					; ja, zurck setzen

_sw_pwm_weiter:

		ldi		ZL, low(PWM_BUFFER)			; Pointer auf RAM-Adresse Puffer setzen
		ldi		ZH, high(PWM_BUFFER)

		in		temp0, PWM_OUT				; Port-Zustand sichern
		sbr		temp0, (1<<PWM_R) | (1<<PWM_G) | (1<<PWM_B) ; Grundzustand PWM an

        ld      temp1, Z+               	; Wert fr Kanal 1 aus dem RAM holen
        cp      pwm_count, temp1        	; Ist der Grenzwert fr Kanal 1 erreicht?
        brlo    _sw_pwm_2                   ; nein, weiter zu Kanal 2
        cbr	    temp0, (1<<PWM_R)       	; ja, LED Kanal 1 aus

_sw_pwm_2:

        ld      temp1, Z+               	; usw.
        cp      pwm_count, temp1
        brlo    _sw_pwm_3
        cbr	    temp0, (1<<PWM_G)

_sw_pwm_3:

        ld      temp1, Z+               	; usw.
		cp      pwm_count, temp1
        brlo    _sw_pwm_out
        cbr	    temp0, (1<<PWM_B)

; Hier ggfs. weitere Kanle einfgen

_sw_pwm_out:

		out		PWM_OUT, temp0				; Bitmuster ausgeben

		pop		ZH							; Register wiederherstellen
		pop		ZL
		pop		temp1
		pop		temp0

		ret									; und raus

; ================================================================================================
