; ************************************************************************************************
; * Kleiner RGB-Controller fr Tiny2313/4313 mit IR-Fernbedienung RC5						 	 *
; * und 3 Kanlen SW-PWM																		 *
; *																								 *
; *	RC5-Empfang, SW-PWM und Zeitbasiserzeugung finden in der selben Timer-ISR statt				 *
; *	angepasst fr 16 MHz Quarz, luft auch mit 8 MHz interner Takt,								 *
; * dazu RC5-Timing und USART-Einstellung anpassen										 		 *
; *																								 *
; *	by Pesi	- Version 1.0 - 02.12.2011															 *
; ************************************************************************************************

.include "tn2313def.inc"					; verwendeter C: Tiny2313

.include "Definitions.inc"					; Definitionen Register, Konstanten etc.

.include "Settings.inc"						; Benutzerdefinierte Einstellungen, Tasten etc.

.CSEG

.include "Interrupts.inc"					; Interrupt-Vektor-Tabelle

; ================================================================================================
; Subroutinen und ISR
; ------------------------------------------------------------------------------------------------

.include "ISR_Timer0OVF.inc"				; Timer0 Overflow -> PWM und RC5 etc.

.include "DMX_Receiver.inc"					; DMX-Empfangsroutine

.include "SW_PWM.inc"						; SW-PWM-Routine

.include "RC5_Receiver.inc"					; Routine fr RC5-Empfang

.include "Get_RC5.inc"						; RC5-Befehlsauswertung

.include "Wait.inc"							; Div. Warteschleifen

.include "Diverses.inc"						; Div. Hilfsroutinen

.include "Key_Command.inc"					; Auswertung empf. Codes

.include "Set_Speed.inc"					; Geschwindigkeit einstellen

.include "Save.inc"							; Einstellungen speichern

.include "Dimmerkorrektur.inc"				; Dimmerwerte umrechnen

; ================================================================================================
; Programmstart
; ------------------------------------------------------------------------------------------------

_reset:

.include "Configuration.inc"				; Ports setzen, Timer konfigurieren etc.

; ================================================================================================
; Hauptschleife
; RC5-Kommandos werden ausgewertet, je nach dem anderes Programm/Farbe eingestellt
; und gedimmt
; ------------------------------------------------------------------------------------------------

_main:

		rcall	_Get_RC5					; Prfen, ob neuer Befehl empfangen

		sbrs	Flags, FL_RC5				; neuer Befehl angekommen?
		rjmp	_run_programm				; nein, weiter im Programm

; ------------------------------------------------------------------------------------------------
; Auswertung der empfangenen Befehle
; manche werden auch spezifisch in den Unterprogrammen behandelt
; ------------------------------------------------------------------------------------------------

		rcall	_Key_Command				; ja, Tastendruck auswerten, Programm einstellen, etc.

_run_programm:

; ------------------------------------------------------------------------------------------------
; Auswahl Programme - Timerabfrage in den Programmen selbst!
; ------------------------------------------------------------------------------------------------

		lds		temp0, PGM_Number			; Programm-Nummer holen

		cpi		temp0, 255					; 255 = "Ausschalten"
		brne	_pgm_select_0
		rjmp	_Off

_pgm_select_0:

		tst		temp0						; 0 = DMX-Empfang
		brne	_pgm_select_1
		rjmp	_DMX

_pgm_select_1:

		cpi		temp0, 1					; 1 = "Feste Farbe"
		brne	_pgm_select_2
		rjmp	_Fixed_Colour

_pgm_select_2:

		cpi		temp0, 2					; 2 = "Baumarktfader"
		brne	_pgm_select_3
		rjmp	_Baumarkt

_pgm_select_3:

		cpi		temp0, 3					; 3 = "Blob"
		brne	_pgm_select_4
		rjmp	_Blob

_pgm_select_4:

		cpi		temp0, 4					; 4 = "Kaminfeuer"
		brne	_pgm_select_5
		rjmp	_Fire

_pgm_select_5:

		cpi		temp0, 5					; 5 = "Zufallsfade"
		brne	_pgm_select_6
		rjmp	_Random_Fade

_pgm_select_6:

		cpi		temp0, 6					; 6 = "Gewitter"
		brne	_pgm_select_x
		rjmp	_Lightning

_pgm_select_x:

		rjmp	_main						; Programm undefiniert, Hauptschleife von vorne

; ------------------------------------------------------------------------------------------------
; Programme (Module), erzeugen Farben und speichern diese im RGB-Buffer
; ------------------------------------------------------------------------------------------------

.include "Off.inc"							; "Ausschalten"
		rjmp	_main						; wieder an, Hauptschleife von vorne

.include "Fixed_Colour.inc"					; "Feste Farbe"
		rjmp	_RGB_Output					; ausgeben

.include "Baumarkt.inc"						; "Baumarktfader"
		rjmp	_RGB_Output					; ausgeben

.include "Blob.inc"							; "Blob"
		rjmp	_RGB_Output					; ausgeben

.include "Fire.inc"							; "Kaminfeuer"
		rjmp	_RGB_Output					; ausgeben

.include "DMX.inc"							; DMX-Empfang
		rjmp	_main						; Ausgabe berspringen, da direkt im Modul

.include "Random_Fade.inc"					; "Zufallsfader"
		rjmp	_RGB_Output					; ausgeben

.include "Lightning.inc"					; "Gewitter"
		rjmp	_RGB_Output					; ausgeben

; ------------------------------------------------------------------------------------------------
; Farben aus RGB-Buffer dimmen und ausgeben
; ------------------------------------------------------------------------------------------------

_RGB_Output:

		lds		temp1, Dimmer				; Dimmer-Wert holen		

		rcall	_Dimmerkorrektur			; und von 0-31 auf 0-255 mit Kurve umrechnen

		ldi		YH, high(RGB_Buffer)		; Zeiger auf Zwischenspeicher
		ldi		YL, low(RGB_Buffer)

		ldi		ZH, high(PWM_Buffer)		; Zeiger auf PWM-Buffer
		ldi		ZL, low(PWM_Buffer)

		ldi		temp2, 3					; 3 Werte bertragen

_rgb_copy_loop:
		
		ld		temp0, Y+					; Wert aus Puffer holen
		rcall	_Dimmer						; dimmen
		st		Z+, temp0					; und speichern

		dec		temp2
		brne	_rgb_copy_loop

		rjmp	_main						; Hauptschleife von vorne

; ================================================================================================
; Daten im Flash und EEPROM
; ------------------------------------------------------------------------------------------------

.include "Data.inc"

; ================================================================================================
